#include <stdio.h>
#include <malloc.h> // χρειάζεται για τις malloc και free
#include <string.h> // χρειάζεται για τις strlen και strcpy

// ορίζω τη σχετική δομή (με το όνομα με πεζά και το πρόθεμα _)
typedef struct _person {
    char *first;        // κάθε πεδίο της δομής να ταιριάζει
    char *last;         // με τον τύπο δεδομένων που δίνεται
    int height;         // και στον constructor
    double weight;
} Person;               // το όνομα του νέου τύπου δεδομένων είναι το ζητούμενο (κεφαλαίο το 1ο γράμμα)

// ο destructor
void prsnDelete(Person *P) {
    // πάντα ελέγχει πρώτα τα δεδομένα και εάν δόθηκε NULL για καταστροφή
    if (P == NULL) {
        // τότε δεν υπάρχει κάπου που να πρέπει να γίνει
        return;
    }

    // ακολουθούν οι αποδεσμεύσεις της σχετικής μνήμης με την αντίστροφη σειρά από αυτά που δεσμεύονται
    // αλλιώς, εάν το όνομα δεν είναι NULL, αποδεσμεύει τη σχετική μνήμη
    if (P->last != NULL) free(P->last);
    // και ομοίως, εάν το επώνυμο δεν είναι NULL, αποδεσμεύει τη σχετική μνήμη
    if (P->first != NULL) free(P->first);

    // τέλος, απελευθερώνεται η μνήμη της ίδιας της δομής, η οποία λόγω του αρχικού ελέγχου σίγουρα δεν είναι NULL
    free(P);
}

// ο constructor
Person *prsnCreate(char *first, char *last, int height, double weight) {
    // ελέγχει τα αρχικά δεδομένα και εάν κάτι δεν πάει καλά επιστρέφει το NULL ως ένδειξη σφάλματος
    if (first == NULL || last == NULL || height <= 0 || weight <= 0.0) {
        return NULL;
    }

    // δεσμεύει μνήμη για τη δομή την ίδια
    Person *P = (Person *)malloc(sizeof(Person));
//  ^^^^^^       ^^^^^^                      ^^^^^^
//  ο τύπος δεδομένων επαναλαμβάνεται 3 φορές,
//  τις δύο πρώτες με ένα * στα δεξιά του περισσότερο από την τρίτη φορά
    // SOS : πάντα μετά τη malloc ελέγχεται η τιμή που επέστρεψε και εάν είναι NULL
    if (P == NULL) {
        // τότε δεν μπορούμε να δημιουργήσουμε καν τη δομή, άρα επιστρέφουμε ένδειξη σφάλματος
        return NULL;
    }

    // δίνουμε αρχικές τιμές στα πεδία της δομής που έχουν native τύπο δεδομένων
    P->weight = weight;
    P->height = height;
    // και NULL σε αυτά που είναι pointers
    P->last = NULL;
    // το first (το ένα από αυτά, αυτό που θα κάνουμε πρώτο malloc) δε χρειάζεται να του δώσουμε
    // αρχική τιμή NULL γιατί θα την πάρει αν είναι από το malloc εάν δεν υπάρχει διαθέσιμη μνήμη
    P->first = (char *)malloc((strlen(first) + 1) * sizeof(char));
    // χρειάζομαι τόσους χαρακτήρες όσους το first ^  + ^ έναν χαρακτήρα για το τερματικό 0

    // SOS : πάντα μετά τη malloc ελέγχεται η τιμή που επέστρεψε και εάν είναι NULL
    if (P->first == NULL) {
        // αποδεσμεύω ότι είχε δεσμευθεί μέχρι τώρα
        prsnDelete(P);
        // και επιστρέφω ένδειξη λάθους
        return NULL;
    }
    // ομοίως, και για το άλλο πεδίο
    P->last = (char *)malloc((1+strlen(last)) * sizeof(char));
    // SOS : πάντα μετά τη malloc ελέγχεται η τιμή που επέστρεψε και εάν είναι NULL
    if (P->last == NULL) {
        // αποδεσμεύω ότι είχε δεσμευθεί μέχρι τώρα
        prsnDelete(P);
        // και επιστρέφω ένδειξη λάθους
        return NULL;
    }

    // τέλος, αφού υπάρχει όλη η απαιτούμενη μνήμη
    // αντιγράφω το όνομα που δόθηκε στη μνήμη που κράτησα γι'αυτό
    strcpy(P->first, first);
    // ομοίως, και το επώνυμο
    strcpy(P->last, last);

    // τέλος, επιστρέφω τον pointer σε αυτή τη δομή
    return P;
}

int main(void) {
    printf("Hello, World!\n");
    return 0;
}
